DsRAutoINV = DsRAutoINV or {}
DsRAutoINV.AddonId = "DsRAutoInvite"
DsRAutoINV.name = "DsRAutoInvite"
------------------------------------------------
--- Utility functions
------------------------------------------------
local function b(v) if v then return "T" else return "F" end end
local function nn(val) if val == nil then return "NIL" else return val end end
local function dbg(msg) if DsRAutoINV.debug then d("|c999999" .. msg) end end
local function echo(msg) CHAT_ROUTER:AddSystemMessage("|CFFFF00" .. msg) end

DsRAutoINV.isCyrodiil = function(unit)
    if unit == nil then unit = "player" end
    dbg("Current zone: '" .. GetUnitZone(unit) .. "'")
    return GetUnitZone(unit) == "Cyrodiil"
end

------------------------------------------------
--- Event handlers
------------------------------------------------

--Main callback fired on chat message
DsRAutoINV.callback = function(_, messageType, from, message)
    if not DsRAutoINV.enabled or not DsRAutoINV.listening then
        return
    end

    --TODO: Move this to the actual invite send so not per-message
    if GetGroupSize() >= DsRAutoINV.cfg.maxSize then
        echo(GetString(SI_DsRAI_GROUP_FULL_STOP))
        DsRAutoINV.stopListening()
    end
    
    if string.lower(message) == DsRAutoINV.cfg.watchStr and from ~= nil and from ~= "" then
        if (messageType >= CHAT_CHANNEL_GUILD_1 and messageType <= CHAT_CHANNEL_OFFICER_5) or messageType == CHAT_CHANNEL_WHISPER then
            from = DsRAutoINV.accountNameLookup(messageType, from)
            if from == "" or from == nil then return end
        end

        echo(zo_strformat(GetString(SI_DsRAI_SEND_TO_USER), from))
        DsRAutoINV:invitePlayer(from)
    end
end


DsRAutoINV.playerLeave = function(_, unitTag, connectStatus, isSelf, acctName, charName)
    local player = GetDisplayName()

    if player == charName then
        if DsRAutoINV.enabled then
            DsRAutoINV.enabled = false
            DsRAutoINV.stopListening()
            DsRAutoINVUI.refresh()
            echo(zo_strformat(GetString(SI_DsRAI_OFF)))
        end
        return;
    else
        if DsRAutoINV.enabled and DsRAutoINV.cfg.restart then
            echo(zo_strformat(GetString(SI_DsRAI_USER_LEAVE), charName))
        if not DsRAutoINV.listening then
            echo(zo_strformat(GetString(SI_DsRAI_GROUP_OPEN_RESTART), DsRAutoINV.cfg.watchStr))
        end
            DsRAutoINV.startListening()
        end

        if isSelf then
            DsRAutoINV.kickTable = {}
        else
            local unitName = GetUnitName(unitTag):gsub("%^.+", "")
             DsRAutoINV.kickTable[unitName] = nil
        end
    end
end

DsRAutoINV.offlineEvent = function(_, unitTag, connectStatus, isSelf, acctName, charName)
    local unitName = GetUnitName(unitTag):gsub("%^.+", "")
    if connectStatus then
        dbg(unitTag .. "/" .. unitName .. " has reconnected")
        DsRAutoINV.kickTable[unitName] = nil
    else
        dbg(unitTag .. "/" .. unitName .. " has disconnected")
        DsRAutoINV.kickTable[unitName] = GetTimeStamp()
    end
    MINI_GROUP_LIST:updateSingle(name)
end

-- tick function: called every 1s
function DsRAutoINV.tick()
    local self = DsRAutoINV
    self.kickCheck()

    if self.listening then
        if GetGroupSize() >= self.cfg.maxSize then
            self.stopListening()
        else
            self:checkSentInvites()
            self:processQueue()
        end
    end
end

------------------------------------------------
--- Event control
------------------------------------------------
DsRAutoINV.disable = function()
    DsRAutoINV.enabled = false
    DsRAutoINV.stopListening()
    echo(zo_strformat(GetString(SI_DsRAI_OFF)))
    EVENT_MANAGER:UnregisterForUpdate(DsRAutoINV.AddonId)
    EVENT_MANAGER:UnregisterForEvent(DsRAutoINV.AddonId, EVENT_GROUP_INVITE_RESPONSE)
end

DsRAutoINV.stopListening = function()
    EVENT_MANAGER:UnregisterForEvent(DsRAutoINV.AddonId, EVENT_CHAT_MESSAGE_CHANNEL)
    DsRAutoINV.listening = false
end

--@param restart: (boolean) - true if restarted listening due to space open up
--currently only used for different print strings
DsRAutoINV.startListening = function(restart)
    if not DsRAutoINV.enabled then
        DsRAutoINV.enabled = true
        DsRAutoINV.checkOffline()
        EVENT_MANAGER:RegisterForUpdate(DsRAutoINV.AddonId, 1000, DsRAutoINV.tick)
        EVENT_MANAGER:RegisterForEvent(DsRAutoINV.AddonId, EVENT_GROUP_INVITE_RESPONSE, DsRAutoINV.inviteResponse)
    end

    if not DsRAutoINV.listening and GetGroupSize() < DsRAutoINV.cfg.maxSize then
        --Add handler
        EVENT_MANAGER:RegisterForEvent(DsRAutoINV.AddonId, EVENT_CHAT_MESSAGE_CHANNEL, DsRAutoINV.callback)
        DsRAutoINV.listening = true
        if restart ~= nil then
            echo(zo_strformat(GetString(SI_DsRAI_GROUP_OPEN_RESTART), DsRAutoINV.cfg.watchStr))
        else
            echo(zo_strformat(GetString(SI_DsRAI_START_ON), DsRAutoINV.cfg.watchStr))
        end
    end
end

------------------------------------------------
--- Initialization
------------------------------------------------
DsRAutoINV.init = function()
    EVENT_MANAGER:UnregisterForEvent("DsRAutoINVInit", EVENT_PLAYER_ACTIVATED)
    if DsRAutoINV.initDone then return end
    DsRAutoINV.initDone = true

    EVENT_MANAGER:RegisterForEvent(DsRAutoINV.AddonId, EVENT_GROUP_MEMBER_LEFT, DsRAutoINV.playerLeave)
    EVENT_MANAGER:RegisterForEvent(DsRAutoINV.AddonId, EVENT_GROUP_MEMBER_CONNECTED_STATUS, DsRAutoINV.offlineEvent)

    --Make sure Offline is updated after player zones (is offline for a bit
    EVENT_MANAGER:RegisterForEvent("DsRAutoINVInit", EVENT_PLAYER_ACTIVATED, DsRAutoINV.checkOffline)

    DsRAutoINV.listening = false
    DsRAutoINV.enabled = false
    DsRAutoINV.player = GetUnitName("player")
    DsRAutoINVUI.init()
end

EVENT_MANAGER:RegisterForEvent("DsRAutoINVInit", EVENT_PLAYER_ACTIVATED, DsRAutoINV.init)
