DsRGuildBeam = {}
local DsRGuildBeam = DsRGuildBeam  or {}

---[ Constants ]---

DsRGuildBeam.Debug = false

DsRGuildBeam.Const = { }
DsRGuildBeam.Const.AddonName = "DsRGuildBeam"
DsRGuildBeam.Const.SavedVarsFile = "DsRGuildRosterSettings"
DsRGuildBeam.Const.SavedVarsDefaults = { }
DsRGuildBeam.Const.DialogAlert = "DsRGuildBeamDialogAlert"
DsRGuildBeam.Const.DialogConfirm = "DsRGuildBeamDialogConfirm"
DsRGuildBeam.Const.DefaultSettings = { }

DsRGuildBeam.Attribute = { }
DsRGuildBeam.Attribute.Active = 1
DsRGuildBeam.Attribute.Dead = 2
DsRGuildBeam.Attribute.Resurrecting = 3
DsRGuildBeam.Attribute.Leader = 4
DsRGuildBeam.Attribute.Healer = 5
DsRGuildBeam.Attribute.Tank = 6
DsRGuildBeam.Attribute.Battleground = 7
DsRGuildBeam.Attribute.Cyrodiil = 8
DsRGuildBeam.Attribute.NonPVP = 9

-- DsRGuildBeam.Color = {
-- 	["Auroran Thunder"] = "",
-- 	["Coral Tower Crimson"] = "", 
-- 	["Colored Room Coral"] = "",
-- 	["Groundskeeper's Green"] = "",
-- 	["Darien's Golden Dawnbreaker"] = "",
-- 	["Dynar's Purple Heart"] = "",
-- 	["Hollowcity Ashen Sky"] = "",
-- 	["Infinite Indigo"] = "",
-- 	["Mauve of Malatar"] = "",
-- 	["Magna-Ge Magenta"] = "",
-- 	["Morningstar Maroon"] = "",
-- 	["Oubliette Orange"] = "",
-- 	["Prismatic Pink (Scented)"] = "",
-- 	["Prismatic Pink (Unscented)"] = "","",
-- }

DsRGuildBeam.RadiantPlayers = { }
DsRGuildBeam.UnitTags = { }

local function InitUnitTags()
	for index = 1, GROUP_SIZE_MAX + 1 do
		if 1 == index then
			DsRGuildBeam.UnitTags[index] = "player"
		else
			DsRGuildBeam.UnitTags[index] = string.format( "group%d", index - 1 )
		end
	end

	DsRGuildBeam.NumUnitTags = #DsRGuildBeam.UnitTags
end
InitUnitTags()

---[ Initialization ]---
function DsRGuildBeam.Initialize()
	DsRGuildBeam.InitVars()
	-- zo_callLater( DsRGuildBeam.InitPlayerToPlayer, 5000 )
end

function DsRGuildBeam.InitVars()
	local savedVars = ZO_SavedVars:NewAccountWide( "DsRGuildRosterSettings", 1, nil, DsRGuildBeam.Const.SavedVarsDefaults ) 

	if not savedVars.Data    then savedVars.Data = { } end
	if not savedVars.Options then savedVars.Options = { } end

	DsRGuildBeam.Vars    = savedVars.Data
	DsRGuildBeam.Options = savedVars.Options
end

function DsRGuildBeam.GetSettingsTable()
	local settings = DsRGuildBeam.Vars.Settings
	if "table" ~= type( settings ) then
		settings = { }
		DsRGuildBeam.Vars.Settings = settings
	end

	return settings
end

function DsRGuildBeam.SetDefaultSetting( settingName, value )
	if "string" == type( settingName ) and "" ~= settingName then
		DsRGuildBeam.Const.DefaultSettings[ settingName ] = value
	end
end

function DsRGuildBeam.GetSetting( settingName, suppressDefault )
	local settings = DsRGuildBeam.GetSettingsTable()
	local value = settings[ settingName ]

	if nil == value and not suppressDefault then value = DsRGuildBeam.Const.DefaultSettings[ settingName ] end
	return value
end

function DsRGuildBeam.SetSetting( settingName, value )
	local settings = DsRGuildBeam.GetSettingsTable()
	settings[ settingName ] = value

	DsRGuildBeam.ResetAllLights()
	return value
end

function DsRGuildBeam.GetColorSetting( settingName, suppressDefault )
	local value = DsRGuildBeam.GetSetting( settingName, suppressDefault )
	if not value or "table" ~= type( value ) then value = { r = 1, g = 1, b = 1 } end
	value.a = 0.7 + 0.3 * zo_clamp( DsRGuildBeam.GetSetting( "LightAlpha" ) or 1, 0.1, 1 )

	return value
end

function DsRGuildBeam.GetColorSettingValues( settingName, suppressDefault )
	local color = DsRGuildBeam.GetColorSetting( settingName, suppressDefault )
	return color.r, color.g, color.b, color.a
end

function DsRGuildBeam.SetColorSetting( settingName, value )
	local settings = DsRGuildBeam.GetSettingsTable()

	if "table" ~= type( value ) then value = { r = 1, g = 1, b = 1 } end
	value.r, value.g, value.b = value.r or 1, value.g or 1, value.b or 1
	settings[ settingName ] = value

	DsRGuildBeam.ResetAllLights()

	return value
end


do
	local DisplayName = GetDisplayName()
	local A = DsRGuildBeam.Attribute
	local Active, Dead, Rez, Lead, Heal, Tank, BG, Cyro, NonPVP = A.Active, A.Dead, A.Resurrecting, A.Leader, A.Healer, A.Tank, A.Battleground, A.Cyrodiil, A.NonPVP
	local WasInGroup = false

	function DsRGuildBeam.OnRefreshLights()
		local effects = DsRGuildBeamEffects.Effect:GetAll()
		if not effects then
			return
		end
		
		if IsPlayerInGroup(DisplayName) then
			WasInGroup = true

			local D = true == DsRGuildBeam.Debug
			local LIGHT_ENABLED = DsRGuildBeam.GetSetting( "LightEnabled" )
			local isBG, isCyro = IsActiveWorldBattleground(), IsInCyrodiil()
			local isNonPVP = not isBG and not isCyro
			local leaderUnitTag = GetGroupLeaderUnitTag()

			for index, effect in ipairs( effects ) do
				if effect and effect.Attributes and not effect.Deleted then
					local attr = effect.Attributes
					local unitTag = effect.UnitTag
					local isPlayer = AreUnitsEqual( "player", unitTag )

					if	( D and isPlayer ) or
						(
							not isPlayer and
							DoesUnitExist( unitTag ) and
							IsUnitOnline( unitTag ) and
							IsGroupMemberInSameWorldAsPlayer( unitTag ) and
							not IsGroupMemberInRemoteRegion( unitTag )
						) then
						local role = GetGroupMemberSelectedRole( unitTag )
						local displayName = string.lower( GetUnitDisplayName( unitTag ) )
						local isRadiant = true == DsRGuildBeam.RadiantPlayers[ displayName ]

						attr[ Active ] = true
						attr[ Dead ] = IsUnitDead( unitTag ) and 1 >= GetUnitPower( unitTag, POWERTYPE_HEALTH )
						attr[ Rez ] = IsUnitBeingResurrected( unitTag ) or DoesUnitHaveResurrectPending( unitTag )
						attr[ Lead ] = leaderUnitTag == unitTag
						attr[ Heal ] = role == LFG_ROLE_HEAL
						attr[ Tank ] = role == LFG_ROLE_TANK
						attr[ BG ] = isBG
						attr[ Cyro ] = isCyro
						attr[ NonPVP ] = isNonPVP
					else
						attr[ Active ] = false
					end

					effect.IsRadiant = false

					if not attr[ Active ] or not LIGHT_ENABLED then
						effect:SetColor( nil, nil, nil, 0 )
					else
						if attr[ Dead ] and DsRGuildBeam.GetSetting( "LightDead" ) then
							if attr[ Rez ] and DsRGuildBeam.GetSetting( "LightResurrecting" ) then
								effect:SetColor( DsRGuildBeam.GetColorSettingValues( "LightResurrectingColor" ) )
							else
								effect:SetColor( DsRGuildBeam.GetColorSettingValues( "LightDeadColor" ) )
							end
						elseif isRadiant then
							effect:SetColor( DsRGuildBeam.GetColorSettingValues( "HighlightRadiantColor" ) )
							effect.IsRadiant = true
						elseif not IsUnitInCombat( "player" ) and DsRGuildBeam.GetSetting( "LightCombatOnly" ) then
							effect:SetColor( nil, nil, nil, 0 )
						elseif attr[ Lead ] and DsRGuildBeam.GetSetting( "HighlightLeader" ) then
							effect:SetColor( DsRGuildBeam.GetColorSettingValues( "HighlightLeaderColor" ) )
						elseif attr[ Heal ] and DsRGuildBeam.GetSetting( "HighlightHealers" ) then
							effect:SetColor( DsRGuildBeam.GetColorSettingValues( "HighlightHealersColor" ) )
						elseif attr[ Tank ] and DsRGuildBeam.GetSetting( "HighlightTanks" ) then
							effect:SetColor( DsRGuildBeam.GetColorSettingValues( "HighlightTanksColor" ) )
						elseif DsRGuildBeam.GetSetting( "LightGroupMembers" ) then
							if attr[ BG ] and DsRGuildBeam.GetSetting( "LightBattlegroundTeam" ) then
								effect:SetColor( DsRGuildBeam.GetColorSettingValues( "LightBattlegroundTeamColor" ) )
							elseif attr[ Cyro ] and DsRGuildBeam.GetSetting( "LightCyrodiilTeam" ) then
								effect:SetColor( DsRGuildBeam.GetColorSettingValues( "LightCyrodiilTeamColor" ) )
							elseif attr[ NonPVP ] and DsRGuildBeam.GetSetting( "LightNonPVPTeam" ) then
								effect:SetColor( DsRGuildBeam.GetColorSettingValues( "LightNonPVPTeamColor" ) )
							end
						else
							effect:SetColor( nil, nil, nil, 0 )
						end
					end
				end
			end
		elseif WasInGroup then
			WasInGroup = false

			for index, effect in ipairs( effects ) do
				effect.Attributes[ Active ] = false
			end
		end
	end
end

function DsRGuildBeam.UnregisterOnUpdate()
	EVENT_MANAGER:UnregisterForUpdate( DsRGuildBeam.Const.AddonName .. "UnregisterOnUpdate" )
end

function DsRGuildBeam.ResetAllLights()
	DsRGuildBeamEffects.Effect:DeleteAll()

	local effect
	for index = 1, DsRGuildBeam.NumUnitTags do
		effect = DsRGuildBeamEffects.Effect:New( "Light of Togetherness" )
		effect.Attributes, effect.UnitTag = { }, DsRGuildBeam.UnitTags[index]
	end

	DsRGuildBeam.OnRefreshLights()
end
