DsRGuildBeamUI = {}
local DsRGuildBeamUI = DsRGuildBeamUI  or {}

DsRGuildBeamUI.UI = DsRGuildBeamUI.UI or { }

---[ Sounds  ]---

do
	local soundReadyTime = 0

	function DsRGuildBeamUI.UI.PlaySound( sound, duration )

		duration = duration or 1000

		local currentTime = GetFrameTimeMilliseconds()
		if currentTime < soundReadyTime then return end

		soundReadyTime = currentTime + duration
		PlaySound( sound )

	end
end

---[ Modal Dialogs ]---

function DsRGuildBeamUI.UI.HideAllDialogs()
	ZO_Dialogs_ReleaseAllDialogs()
end

function DsRGuildBeamUI.UI.SetupAlertDialog()
    if not ESO_Dialogs[ DsRGuildBeam.Const.DialogAlert ] then
		ESO_Dialogs[ DsRGuildBeam.Const.DialogAlert ] = {
            canQueue = true,
            title = {
                text = "",
            },
            mainText = {
                text = "",
            },
            buttons = {
                [1] = {
                    text = SI_OK,
                    callback = function( dialog ) end,
                },
            }
        }
    end

	return ESO_Dialogs[ DsRGuildBeam.Const.DialogAlert ]
end

function DsRGuildBeamUI.UI.ShowAlertDialog( message, confirmCallback )
    local dialog = DsRGuildBeamUI.UI.SetupAlertDialog()
    dialog.title.text = "DsR GuildRoster"
    dialog.mainText.text = message
    dialog.buttons[1].callback = function()
		if nil ~= confirmCallback then
			confirmCallback()
		end
	end

    ZO_Dialogs_ShowDialog( DsRGuildBeam.Const.DialogAlert )
end

function DsRGuildBeamUI.UI.SetupConfirmDialog()
    if not ESO_Dialogs[ DsRGuildBeam.Const.DialogConfirm ] then
		ESO_Dialogs[ DsRGuildBeam.Const.DialogConfirm ] = {
            canQueue = true,
            title = {
                text = "",
            },
            mainText = {
                text = "",
            },
            buttons = {
                [1] = {
                    text = SI_DIALOG_CONFIRM,
                    callback = function( dialog ) end,
                },
                [2] = {
                    text = SI_DIALOG_CANCEL,
					callback = function( dialog ) end,
                }
            }
        }
    end

	return ESO_Dialogs[ DsRGuildBeam.Const.DialogConfirm ]
end

function DsRGuildBeamUI.UI.ShowConfirmationDialog( title, body, confirmCallback, cancelCallback )
    local dialog = DsRGuildBeamUI.UI.SetupConfirmDialog()
    dialog.title.text = "DsR GuildRoster"
    dialog.mainText.text = body
    dialog.buttons[1].callback = function()
		if nil ~= confirmCallback then
			confirmCallback()
		end
	end
	dialog.buttons[2].callback = function()
		if nil ~= cancelCallback then
			cancelCallback()
		end
	end

    ZO_Dialogs_ShowDialog( DsRGuildBeam.Const.DialogConfirm )
end

---[ Notification ]---

function DsRGuildBeamUI.UI.SetupNotificationDialog()
	local ui = DsRGuildBeamUI.UI.NotificationDialog

	if nil == ui then
		ui = { }
		DsRGuildBeamUI.UI.NotificationDialog = ui

		local prefix = "DsRGuildBeamUINotificationDialog"
		local c, grp, win

		-- Window

		win = WINDOW_MANAGER:CreateTopLevelWindow( prefix )
		ui.Window = win
		win:SetAlpha( 0.5 )
		win:SetClampedToScreen( true )
		win:SetMouseEnabled( false )
		win:SetMovable( false )
		win:SetResizeHandleSize( 0 )
		win:SetAnchor( BOTTOM, GuiRoot, BOTTOM, 0, -340 )
		win:SetDimensions( 1, 100 )
		win:SetHidden( true )

		-- Controls

		c = WINDOW_MANAGER:CreateControl( prefix .. "Message", win, CT_LABEL )
		ui.Message = c
		c:SetColor( 1, 1, 0.5, 1 )
		c:SetText( "" )
		c:SetFont( "$(BOLD_FONT)|$(KB_32)|soft-shadow-thick" )
		c:SetHorizontalAlignment( TEXT_ALIGN_CENTER )
		c:SetVerticalAlignment( TEXT_ALIGN_TOP )
		c:SetAnchor( CENTER, win, CENTER, 0, 0 )
	end

	return ui
end

function DsRGuildBeamUI.UI.HideNotification()
	local ui = DsRGuildBeamUI.UI.SetupNotificationDialog()
	ui.Window:SetHidden( true )
end

function DsRGuildBeamUI.UI.FadeNotification()
	local ui = DsRGuildBeamUI.UI.NotificationDialog
	local alpha = ui.CurrentAlpha

	if not ui.IsFading then
		if 1 > alpha then
			alpha = alpha + 0.01
		else
			ui.IsFading = true
		end
	else
		alpha = alpha - 0.01
	end

	ui.Window:SetAlpha( alpha )
	ui.CurrentAlpha = alpha

	if ui.IsFading and 0.01 > alpha then
		DsRGuildBeamUI.UI.HideNotification()
		EVENT_MANAGER:UnregisterForUpdate( DsRGuildBeam.Const.AddonName .. "FadeNotification" )
	end
end

function DsRGuildBeamUI.UI.DisplayNotification( message )
	local ui = DsRGuildBeamUI.UI.SetupNotificationDialog()
	ui.Message:SetText( message )
	ui.Window:SetHidden( false )
	ui.Window:SetAlpha( 0 )
	ui.CurrentAlpha = 0
	ui.IsFading = false

	EVENT_MANAGER:RegisterForUpdate( DsRGuildBeam.Const.AddonName .. "FadeNotification", 20, DsRGuildBeamUI.UI.FadeNotification )
end
