-- Create namespace
DsRGuildAchievTracker = {}
local DsRGuildAchievTracker = DsRGuildAchievTracker or {}

DsRGuildAchievTracker.name = "DsRGuildAchievTracker"

DsRGuildAchievTracker.ACCdefaults = {
    showDetails      = {},
    positionoffsetX  = 100,
    positionoffsetY  = 100,
    fontSizename     = 16,
    fontSizedesc     = 14,
    sizeX            = 100,
    sizeY            = 200,
    locked           = false,
    showIcons        = true,
    showDesc         = false,
    showTrackedAchievementCount = true,
    autoTrackZoneAchievements   = false,
    hideOldZoneAchievements     = true,
    hideCompleted    = true,
    bgAlpha          = 100,
    maxTracked       = 0,
    hidden           = false,
    AchievTrackOnOff = false,
}
DsRGuildAchievTracker.CHARdefaults = {
    tracked     = {}
}

DsRGuildAchievTracker.config               = nil
DsRGuildAchievTracker.configCHAR           = nil
DsRGuildAchievTracker.lastZone             = nil
DsRGuildAchievTracker.hiddenShortly        = false
DsRGuildAchievTracker.checkboxesCreated    = false
DsRGuildAchievTracker.heightPerLine        = 50
DsRGuildAchievTracker.bufferTable          = {}

local function comma_value(amount)
  local formatted = amount
  while true do
    formatted, k = string.gsub(formatted, "^(-?%d+)(%d%d%d)", '%1,%2')
    if (k==0) then
      break
    end
  end
  return formatted
end

function DsRGuildAchievTracker:Buffer(key, buffer)
  	if key == nil then return end
  	if DsRGuildAchievTracker.bufferTable[key] == nil then DsRGuildAchievTracker.bufferTable[key] = {} end
  	DsRGuildAchievTracker.bufferTable[key].buffer = buffer or 3
  	DsRGuildAchievTracker.bufferTable[key].now = GetFrameTimeMilliseconds()
  	if DsRGuildAchievTracker.bufferTable[key].last == nil then DsRGuildAchievTracker.bufferTable[key].last = DsRGuildAchievTracker.bufferTable[key].now end
  	DsRGuildAchievTracker.bufferTable[key].diff = DsRGuildAchievTracker.bufferTable[key].now - DsRGuildAchievTracker.bufferTable[key].last
  	DsRGuildAchievTracker.bufferTable[key].eval = DsRGuildAchievTracker.bufferTable[key].diff >= DsRGuildAchievTracker.bufferTable[key].buffer
  	if DsRGuildAchievTracker.bufferTable[key].eval then DsRGuildAchievTracker.bufferTable[key].last = DsRGuildAchievTracker.bufferTable[key].now end
  	return DsRGuildAchievTracker.bufferTable[key].eval
end

function DsRGuildAchievTracker:GetLabel(name, parent, isBold)
  	local label = parent:GetNamedChild(name) or WINDOW_MANAGER:CreateControl(parent:GetName() .. name, parent, CT_LABEL)
  	label:SetHidden(false)
  	label:SetDimensions(250, 25)
  	label:SetWrapMode(TEXT_WRAP_MODE_ELLIPSIS)

  	local fontFile = nil
  	local fontSize = DsRGuildAchievTracker.config.fontSizedesc
  	local fontDecoration = "soft-shadow-thin"
  	if isBold then
    	fontSize = DsRGuildAchievTracker.config.fontSizename
    	fontFile = ZoFontGameBold:GetFontInfo()
  	else
    	fontFile = ZoFontGame:GetFontInfo()
  	end

  	label:SetFont(string.format("%s|%d|%s", fontFile, fontSize, fontDecoration))
  	return label
end

function DsRGuildAchievTracker:GetIcon(name, parent, texture)
  	local ico = parent:GetNamedChild(name) or WINDOW_MANAGER:CreateControl(parent:GetName() .. name, parent, CT_TEXTURE)
  	ico:SetHidden(false)
  	ico:SetDimensions(25, 25)
  	ico:SetTexture(texture)
  	return ico
end

function DsRGuildAchievTracker:AutoTrackZoneAchievements()
  	if DsRGuildAchievTracker.config == nil then return end

  	if DsRGuildAchievTracker.config.autoTrackZoneAchievements then
    	local currentZone = GetUnitZone("player")
    	if currentZone ~= DsRGuildAchievTracker.lastZone or DsRGuildAchievTracker.lastZone == nil then
      		if DsRGuildAchievTracker.lastZone ~= nil and DsRGuildAchievTracker.config.hideOldZoneAchievements then
        		DsRGuildAchievTracker:FindZoneAchievements(DsRGuildAchievTracker.lastZone, false)
      		end
      	DsRGuildAchievTracker:FindZoneAchievements(currentZone, true)
      	DsRGuildAchievTracker.lastZone = currentZone
    	end
  	end
end

function DsRGuildAchievTracker:FindZoneAchievements(zone, track)
  	local numCats = GetNumAchievementCategories()

  	local function FindAndTrack(cat, subCat, index)
        local id = GetAchievementId(cat, subCat, index)
        local name, desc = GetAchievementInfo(id)
        if string.find(name, zone) ~= nil or string.find(desc, zone) ~= nil then
          	if track then
            	DsRGuildAchievTracker.configCHAR.tracked[id] = true
            	DsRGuildAchievTracker.config.showDetails[id] = false
          	else
            	DsRGuildAchievTracker.configCHAR.tracked[id] = nil
            	DsRGuildAchievTracker.config.showDetails[id] = nil
          	end
        end
    end

    for iCat = 1, numCats do
  	    local _, numSubCats, numAchievs = GetAchievementCategoryInfo(iCat)
        if numSubCats > 0 then
      	    for iSubCat = 1, numSubCats do
        	    local _, numAchievs = GetAchievementSubCategoryInfo(iCat, iSubCat)
        	    for iAchiev = 1, numAchievs do
        		    FindAndTrack(iCat, iSubCat, iAchiev)
        	    end
      	    end
        else
            for iAchiev = 1, numAchievs do
        	    FindAndTrack(iCat, nil, iAchiev)
            end
        end
    end
end

function DsRGuildAchievTracker:CreateWindow()
  	DsRGuildAchievTracker.frame = WINDOW_MANAGER:CreateTopLevelWindow("DsRGuildAchievTrackerWindow")

  	DsRGuildAchievTracker.frame:SetAnchor(TOPLEFT, GuiRoot, TOPLEFT, DsRGuildAchievTracker.config.positionoffsetX, DsRGuildAchievTracker.config.positionoffsetY)
  	DsRGuildAchievTracker.frame:SetHidden(false)
  	DsRGuildAchievTracker.frame:SetMovable(not DsRGuildAchievTracker.config.locked)
  	DsRGuildAchievTracker.frame:SetMouseEnabled(true)
  	DsRGuildAchievTracker.frame:SetResizeToFitDescendents(true)

  	DsRGuildAchievTracker.frame:SetResizeHandleSize(MOUSE_CURSOR_RESIZE_NS)
  	DsRGuildAchievTracker.frame:SetHandler("OnMouseUp", function()
    	local sizeX = DsRGuildAchievTracker.frame:GetWidth()
    	local sizeY = DsRGuildAchievTracker.frame:GetHeight()
    	DsRGuildAchievTracker.frame:SetDimensions(sizeX, sizeY)
    	DsRGuildAchievTracker.config.positionoffsetX = DsRGuildAchievTracker.frame:GetLeft()
    	DsRGuildAchievTracker.config.positionoffsetY = DsRGuildAchievTracker.frame:GetTop()
  	end)

  	DsRGuildAchievTracker.frame.bg = WINDOW_MANAGER:CreateControl(nil, DsRGuildAchievTracker.frame, CT_BACKDROP)
  	DsRGuildAchievTracker.frame.bg:SetAnchorFill(DsRGuildAchievTracker.frame)
  	DsRGuildAchievTracker.frame.bg:SetCenterColor(0.0, 0.0, 0.0, 0.25)
  	DsRGuildAchievTracker.frame.bg:SetEdgeColor(0.0, 0.0, 0.0, 0.3)
  	DsRGuildAchievTracker.frame.bg:SetEdgeTexture(nil, 2, 2, 2.0, 2.0)
  	DsRGuildAchievTracker.frame.bg:SetDrawLayer(DL_BACKGROUND)
end

function DsRGuildAchievTracker:LoadTrackedAchievements(updatedId)
    DsRGuildAchievTracker.frame.bg:SetAlpha(self.config.bgAlpha / 100)
        
  	if DsRGuildAchievTracker.config.locked then
    	DsRGuildAchievTracker.frame:SetMovable(false)
  	else
    	DsRGuildAchievTracker.frame:SetMovable(true)
  	end

  	if DsRGuildAchievTracker.config.autoTrackZoneAchievements then
    	DsRGuildAchievTracker:AutoTrackZoneAchievements(GetUnitZone("player"))
  	end

  	if DsRGuildAchievTracker.hiddenShortly or DsRGuildAchievTracker.config.hidden then
    	DsRGuildAchievTracker.frame:SetHidden(true)
  	else
    	DsRGuildAchievTracker.frame:SetHidden(false)
  	end
    -- hide children and adjust size of the frame
    local numChildren = DsRGuildAchievTracker.frame:GetNumChildren()
    for i = 1, numChildren do
        local child = DsRGuildAchievTracker.frame:GetChild(i)
        if child then
            local height = child:GetHeight()
            child:SetHidden(true)
            child:SetDimensions(0, 0)
            DsRGuildAchievTracker.frame:SetHeight(DsRGuildAchievTracker.frame:GetHeight() - height)
        end
    end

    -- no need to do all the stuff below if frame is hidden
    if DsRGuildAchievTracker.frame:IsHidden() then return end

    local i = 1
    local lastTotalHeight = 0;

    -- traverse tracked achievs and display them
    for id, isTracked in pairs(DsRGuildAchievTracker.configCHAR.tracked) do
        local name, desc, _, icon, done = GetAchievementInfo(id)
        local cName = zo_strformat(SI_TOOLTIP_ITEM_NAME, name)
        local cDesc = zo_strformat(SI_TOOLTIP_ITEM_NAME, desc)

        -- check if done
        local continue = true
        if done and DsRGuildAchievTracker.config.hideCompleted then
            continue = false
            DsRGuildAchievTracker.configCHAR.tracked[id] = nil
            DsRGuildAchievTracker.config.showDetails[id] = nil
        end

        local max = DsRGuildAchievTracker.config.maxTracked
        if DsRGuildAchievTracker.config.maxTracked == 0 then max = 9999 end

        if i > max then return end

        if isTracked and continue then
            local offsetY = 5
            if lastTotalHeight > 0 then offsetY = lastTotalHeight end

            -- icon
            local achievIcon = DsRGuildAchievTracker:GetIcon("Icon" .. i, DsRGuildAchievTracker.frame, icon)
            achievIcon:SetAnchor(TOPLEFT, DsRGuildAchievTracker.frame, TOPLEFT, 5, offsetY)
            achievIcon:SetHidden(not DsRGuildAchievTracker.config.showIcons)

            -- name
            local achievName = DsRGuildAchievTracker:GetLabel("Label" .. i, DsRGuildAchievTracker.frame, true)
            local xPos = 5
            if achievIcon:IsHidden() then xPos = -15 end
            achievName:SetHeight(15)
            achievName:SetAnchor(TOPLEFT, achievIcon, TOPRIGHT, xPos, 0)
            achievName:SetText(cName)
            achievName:SetMouseEnabled(true)
            if done then
                achievName:SetColor(0, 1, 0, 1)
            else
                achievName:SetColor(1, 1, 1, 1)
            end

            achievName:SetHandler("OnMouseEnter", function(self)
                achievName:SetColor(1, 0.86, 0, 1)
            end)
            achievName:SetHandler("OnMouseExit", function(self)
                if done then
                    achievName:SetColor(0, 1, 0, 1)
                else
                    achievName:SetColor(1, 1, 1, 1)
                end
            end)
            achievName:SetHandler("OnMouseUp", function(_, button)
                if button == 1 then
                    local categoryIndex, subCategoryIndex, achievementIndex = GetCategoryInfoFromAchievementId(id)
                    SCENE_MANAGER:Show("achievements")
                    ACHIEVEMENTS:OpenCategory(categoryIndex, subCategoryIndex)
                elseif button == 3 then
                    DsRGuildAchievTracker.configCHAR.tracked[id] = nil
                    DsRGuildAchievTracker.config.showDetails[id] = nil
                    return DsRGuildAchievTracker:LoadTrackedAchievements()
                elseif button == 2 then
                    DsRGuildAchievTracker.config.showDetails[id] = not DsRGuildAchievTracker.config.showDetails[id]
                    return DsRGuildAchievTracker:LoadTrackedAchievements()
                end
            end)

            -- description
            local achievDesc = DsRGuildAchievTracker:GetLabel("Desc" .. i, DsRGuildAchievTracker.frame)
            achievDesc:SetHeight(100)
            achievDesc:SetAnchor(TOPLEFT, achievName, TOPLEFT, 10, achievName:GetTextHeight())
            if done then
                achievDesc:SetColor(0, 1, 0, 1)
            else
                achievDesc:SetColor(0.8, 0.8, 0.8, 1)
            end

            if DsRGuildAchievTracker.config.showDesc then
                achievDesc:SetText(cDesc)
            else
                achievDesc:SetText("")
            end

            -- criteria
            local achievCriteria = DsRGuildAchievTracker:GetLabel("Criteria" .. i, DsRGuildAchievTracker.frame)
            achievCriteria:SetAnchor(TOPLEFT, achievDesc, TOPLEFT, 0, achievDesc:GetTextHeight())
            if done then
                achievCriteria:SetColor(0, 1, 0, 1)
            else
                achievCriteria:SetColor(0.8, 0.8, 0.8, 1)
            end

            local numCriteria = GetAchievementNumCriteria(id)
            local totalCompleted = 0
            local totalRequired = 0

            if DsRGuildAchievTracker.config.showDetails[id] then
                local text = ""
                local critDisplayed = 0
                for criteria = 1, numCriteria do
                local critDesc, critCompleted, critRequired = GetAchievementCriterion(id, criteria)

                if (critCompleted ~= critRequired) or (critCompleted == critRequired and not DsRGuildAchievTracker.config.hideCompleted) then
                    critDisplayed = critDisplayed + 1
                    if critDisplayed > 1 then
                        text = text .. "\n"
                    end

                    if critCompleted ~= critRequired then
                        if critRequired == 1 then
                            text = text .. " - " .. critDesc
                        else
                            text = text .. " - " .. critDesc .. " : " .. critCompleted .. "/" .. critRequired
                        end
                    else
                        text = text .. "|c2ECC40 - " .. critDesc .. "|r"
                    end
                end
            end
            achievCriteria:SetText(text)
            achievCriteria:SetHeight(achievCriteria:GetTextHeight() * numCriteria)
            DsRGuildAchievTracker.frame:SetHeight(achievCriteria:GetTextHeight() * numCriteria)
        else
            for criteria = 1, numCriteria do
                local critDesc, critCompleted, critRequired = GetAchievementCriterion(id, criteria)
                totalCompleted = totalCompleted + critCompleted
                totalRequired = totalRequired + critRequired
            end
            if totalCompleted > 1 then
                totalCompleted = comma_value(totalCompleted)
            end
            if totalRequired > 1 then
                    totalRequired = comma_value(totalRequired)
                    achievCriteria:SetText(" - " .. totalCompleted .. "/" .. totalRequired)
                else
                    achievCriteria:SetText("")
                end
            end
            lastTotalHeight = lastTotalHeight + achievName:GetTextHeight() + achievDesc:GetTextHeight() + achievCriteria:GetTextHeight() + 10
            i = i + 1
            DsRGuildAchievTracker.frame:SetHeight(lastTotalHeight)
        end
    end
end

function DsRGuildAchievTracker:AddTrackedAchievementCount()
    if not DsRGuildAchievTracker:Buffer("AddTrackedAchievementCount", .5) then return end

    local parent = ZO_AchievementsContentsCategoriesScrollChildContainer1
    local numChildren = parent:GetNumChildren()

    local AppendTrackedCount = function(control, trackCount)
        local nm = control:GetName().."trackercount"
        AchievCounter = _G[nm] or WINDOW_MANAGER:CreateControl(nm, control, CT_LABEL)
        if trackCount > 0 then
            AchievCounter:SetText(tostring("|cFFFF00" .. trackCount .."|r"))
        else
            AchievCounter:SetText(tostring("|cAAAAAA".. trackCount .. "|r"))
        end
		AchievCounter:SetWrapMode( TEXT_WRAP_MODE_ELLIPSIS )
		AchievCounter:SetAnchor( RIGHT, control, LEFT, -3, 0 )
		AchievCounter:SetHorizontalAlignment(TEXT_ALIGN_RIGHT)
		AchievCounter:SetVerticalAlignment(TEXT_ALIGN_CENTER)
	    AchievCounter:SetFont(string.format("%s|%d|%s", ZoFontGameBold:GetFontInfo(), 18, "soft-shadow-thin"))
    end

	local FindTrackedForSubcat = function(subcatControl, catIndex, subcatIndex)
		subcatControl.tracked = 0
		if subcatControl.orgText == nil then
            subcatControl.orgText = subcatControl:GetText()
        end
		for id, isTracked in pairs(DsRGuildAchievTracker.configCHAR.tracked) do
			if isTracked then
				local _catIndex, _subcatIndex, _ = GetCategoryInfoFromAchievementId(id)
				if catIndex == _catIndex and subcatIndex == _subcatIndex then
                    subcatControl.tracked = subcatControl.tracked + 1
                end
			end
		end
	end
    for i = 2, numChildren do
        local child = parent:GetChild(i)
        local txtChild = child:GetNamedChild("Text") or nil
                
        if txtChild ~= nil then
            if txtChild.orgText == nil then
                txtChild.orgText = txtChild:GetText()
            end
            local tracked = 0
            for id, isTracked in pairs(DsRGuildAchievTracker.configCHAR.tracked) do
                if isTracked then
                    local catIndex, subCatIndex, achievementIndex = GetCategoryInfoFromAchievementId(id)
                    if child.node.data.categoryIndex == catIndex then
                        tracked = tracked + 1
                    end
                end
            end
            AppendTrackedCount(txtChild, tracked)
        else
            local numSubcats = child:GetNumChildren()
            for j = 1, numSubcats do
                local subcat = child:GetChild(j)
                if j == 1 then
                    FindTrackedForSubcat(subcat, subcat.node.data.parentData.categoryIndex, nil)
                else
                    FindTrackedForSubcat(subcat, subcat.node.data.parentData.categoryIndex, j - 1)
                end
                AppendTrackedCount(subcat, subcat.tracked)
            end
        end
    end
end

function DsRGuildAchievTracker:CreateCheckboxes()
    local list = ZO_AchievementsContentsContentListScrollChild
    local numOfAchievements = list:GetNumChildren()

    for i = 1, numOfAchievements do
        local achievement = list:GetChild(i)
        if achievement ~= nil then
            local title = achievement:GetNamedChild("Title")
            if title ~= nil then
                title:SetAnchor(3, achievement, 3, 110, 10)

                local id = achievement["achievement"]["achievementId"]

                local controlName = tostring(title:GetName() .. "TrackerCheckbox")

                local cb = title:GetNamedChild("TrackerCheckbox") or WINDOW_MANAGER:CreateControlFromVirtual(controlName, title, "ZO_CheckButton")
                cb:SetAnchor(LEFT, title, LEFT, -20, 0)
                cb:SetDimensions(15, 15)
                cb:SetHidden(false)
                cb:SetHandler("OnMouseUp", function() DsRGuildAchievTracker:ToggleTracked() end)

                if DsRGuildAchievTracker.configCHAR.tracked[id] == true then
                    state = BSTATE_PRESSED
                else
                    state = BSTATE_NORMAL
                end

                cb:SetState(state, false)
            end
        end
    end
end

function DsRGuildAchievTracker:ToggleWindow()
    DsRGuildAchievTracker.config.hidden = not DsRGuildAchievTracker.config.hidden
    DsRGuildAchievTracker:LoadTrackedAchievements()
end

function DsRGuildAchievTracker:ToggleTracked()
    local control = WINDOW_MANAGER:GetMouseOverControl()
    local firer   = control:GetParent():GetParent()
    local id      = firer["achievement"]["achievementId"]

    if DsRGuildAchievTracker.configCHAR.tracked[id] then
        DsRGuildAchievTracker.configCHAR.tracked[id] = nil
        DsRGuildAchievTracker.config.showDetails[id] = nil
    else
        DsRGuildAchievTracker.configCHAR.tracked[id] = true
    end

    DsRGuildAchievTracker:LoadTrackedAchievements()
end

function DsRGuildAchievTracker:CheckAchievementJournalVisibility()
    if not DsRGuildAchievTracker:Buffer("CheckAchievementJournalVisibility", .5) then return end

    if ZO_AchievementsContentsContentListScrollChild:IsHidden() == false then
        DsRGuildAchievTracker:CreateCheckboxes()
    end
end

function DsRGuildAchievTracker:Reset()
    for id, isTracked in pairs(DsRGuildAchievTracker.configCHAR.tracked) do
        DsRGuildAchievTracker.configCHAR.tracked[id] = nil
        DsRGuildAchievTracker.config.showDetails[id] = nil
    end
    DsRGuildAchievTracker:LoadTrackedAchievements()
end

function DsRGuildAchievTracker:GetFrame()
    return DsRGuildAchievTracker.frame
end

function DsRGuildAchievTracker:GetIsHiddenShortly()
    return DsRGuildAchievTracker.hiddenShortly
end

function DsRGuildAchievTracker:SetHiddenShortly(hidden)
    local oldStatus = DsRGuildAchievTracker.hiddenShortly
    DsRGuildAchievTracker.hiddenShortly = hidden
    if DsRGuildAchievTracker.hiddenShortly ~= oldStatus then DsRGuildAchievTracker:LoadTrackedAchievements() end
end

function DsRGuildAchievTracker.achieveSceneChange(oldState, newState)
    if newState == SCENE_SHOWN then
    if DsRGuildAchievTracker.oldAchievementSetupFunction==nil then
        DsRGuildAchievTracker.oldAchievementSetupFunction=ACHIEVEMENTS.categoryTree.templateInfo.ZO_TreeLabelSubCategory.setupFunction
      ACHIEVEMENTS.categoryTree.templateInfo.ZO_TreeLabelSubCategory.setupFunction = DsRGuildAchievTracker.AchievementSetupFunction
      ACHIEVEMENTS.refreshGroups:RefreshAll("FullUpdate")
    end
    end
end

function DsRGuildAchievTracker.AchievementSetupFunction(node, control, data, open, userRequested, enabled)
    DsRGuildAchievTracker.oldAchievementSetupFunction(node,control,data,open,userRequested,enabled)
    -- >>> Farbe berechnen
    local numAch, subEarned, subTotal
    local parentData = data.parentData
    if not data.isFakedSubcategory and parentData then
        numAch,subEarned,subTotal = select(2,GetAchievementSubCategoryInfo(parentData.categoryIndex, data.categoryIndex))
    else
        local numSubCat,numAchie,earnedPnt,totalPnt,hidesP = select(2,GetAchievementCategoryInfo(data.categoryIndex))
        if parentData then
            for subCatIndex = 1, numSubCat do
                local subCatEarned,subCatTotal = select(3,GetAchievementSubCategoryInfo(parentData.categoryIndex, subCatIndex))
                earnedPnt = earnedPnt - subCatEarned
                totalPnt = totalPnt - subCatTotal
            end
        end
        subEarned = earnedPnt
        subTotal = totalPnt
    end
    local col = (subEarned == subTotal) and DsRGuildAchievTracker.col_grn or DsRGuildAchievTracker.col_red
    -- Farbe berechnen <<<
    ZO_SelectableLabel_SetNormalColor(control, col)
    if control.GetTextColor ~= DsRGuildAchievTracker.GetTextColor then control.GetTextColor = DsRGuildAchievTracker.GetTextColor end
    control:RefreshTextColor()
    local subMiss = subTotal - subEarned
    if subMiss > 0 then
        local oldTxt = control:GetText()
        control:SetFont(string.format("%s|%d|%s", ZoFontGameBold:GetFontInfo(), 18, "soft-shadow-thin"))
        control:SetText(string.format("%s (%s)", tostring(oldTxt), tostring(subMiss)))
    end
end

function DsRGuildAchievTracker:GetTextColor()
    local b,c,d,e=self.normalColor:UnpackRGBA()
    if self.selected then return b,c,d,0.4 elseif self.mouseover then return b,c,d,0.7 end;
    return b,c,d,e 
end

-------------------------------------------------------------------------------------------------------------------------------------------------
-- ON ADDON LAODED
-------------------------------------------------------------------------------------------------------------------------------------------------
function DsRGuildAchievTracker.OnAddonLoaded(event, name)
    
    DsRGuildAchievTracker.config     = ZO_SavedVars:NewAccountWide("DsRGuildAchievTrackerSettings", 1, nil, DsRGuildAchievTracker.ACCdefaults)
    DsRGuildAchievTracker.configCHAR = ZO_SavedVars:New("DsRGuildAchievTrackerSettings", 1, nil, DsRGuildAchievTracker.CHARdefaults)
    
    if DsRGuildAchievTracker.config.AchievTrackOnOff == true then return end
    
    ZO_Achievements:SetHandler("OnUpdate", function()
        DsRGuildAchievTracker:CheckAchievementJournalVisibility()
        if DsRGuildAchievTracker.config.showTrackedAchievementCount then
            DsRGuildAchievTracker:AddTrackedAchievementCount()
        end
    end)
    DsRGuildAchievTracker:CreateWindow()

    DsRGuildAchievTracker.col_grn = ZO_ColorDef:New("66ff66")
	DsRGuildAchievTracker.col_red = ZO_ColorDef:New("ff6666")
    
    SCENE_MANAGER:GetScene("achievements"):RegisterCallback("StateChange", DsRGuildAchievTracker.achieveSceneChange)

    -- hide listeners
    ZO_PreHookHandler(ZO_MainMenuCategoryBar, "OnShow", function()
        DsRGuildAchievTracker:SetHiddenShortly(true)
    end)
    ZO_PreHookHandler(ZO_InteractWindow, "OnShow", function()
        DsRGuildAchievTracker:SetHiddenShortly(true)
    end)
    ZO_PreHookHandler(ZO_GameMenu_InGame, "OnShow", function()
        DsRGuildAchievTracker:SetHiddenShortly(true)
    end)
    ZO_PreHookHandler(ZO_KeybindStripControl, "OnShow", function()
        DsRGuildAchievTracker:SetHiddenShortly(true)
    end)

    -- show listeners
    ZO_PreHookHandler(ZO_MainMenuCategoryBar, "OnHide", function()
        DsRGuildAchievTracker:SetHiddenShortly(false)
    end)
    ZO_PreHookHandler(ZO_InteractWindow, "OnHide", function()
        DsRGuildAchievTracker:SetHiddenShortly(false)
    end)
    ZO_PreHookHandler(ZO_GameMenu_InGame, "OnHide", function()
        DsRGuildAchievTracker:SetHiddenShortly(false)
    end)
    ZO_PreHookHandler(ZO_KeybindStripControl, "OnHide", function()
        DsRGuildAchievTracker:SetHiddenShortly(false)
    end)

    -- event hooks
    DsRGuildAchievTrackerWindow:RegisterForEvent(EVENT_ZONE_CHANGED, function() DsRGuildAchievTracker:AutoTrackZoneAchievements() end)
    DsRGuildAchievTrackerWindow:RegisterForEvent(EVENT_ACHIEVEMENT_UPDATED, function(_, id) DsRGuildAchievTracker:LoadTrackedAchievements(id) end)
    
    EVENT_MANAGER:UnregisterForEvent ("DsRGuildAchievTracker", EVENT_ADD_ON_LOADED )
    
    DsRGuildAchievTracker:LoadTrackedAchievements()
end
